import { RsbuildPlugin } from '@rsbuild/core';
import { ChainedConfigWithUtils, NormalizedConfig, BundlerChain, ChainIdentifier } from '@rsbuild/shared';
import { PluginItem, TransformOptions } from '@babel/core';
export { TransformOptions as BabelTransformOptions } from '@babel/core';

type PresetEnvTargets = string | string[] | Record<string, string>;
type PresetEnvBuiltIns = 'usage' | 'entry' | false;
type PresetEnvOptions = {
    targets?: PresetEnvTargets;
    bugfixes?: boolean;
    spec?: boolean;
    loose?: boolean;
    modules?: 'amd' | 'umd' | 'systemjs' | 'commonjs' | 'cjs' | 'auto' | false;
    debug?: boolean;
    include?: string[];
    exclude?: string[];
    useBuiltIns?: PresetEnvBuiltIns;
    corejs?: string | {
        version: string;
        proposals: boolean;
    };
    forceAllTransforms?: boolean;
    configPath?: string;
    ignoreBrowserslistConfig?: boolean;
    browserslistEnv?: string;
    shippedProposals?: boolean;
};
interface SharedBabelPresetReactOptions {
    development?: boolean;
    throwIfNamespace?: boolean;
}
interface AutomaticRuntimePresetReactOptions extends SharedBabelPresetReactOptions {
    runtime?: 'automatic';
    importSource?: string;
}
interface ClassicRuntimePresetReactOptions extends SharedBabelPresetReactOptions {
    runtime?: 'classic';
    pragma?: string;
    pragmaFrag?: string;
    useBuiltIns?: boolean;
    useSpread?: boolean;
}
type PresetReactOptions = AutomaticRuntimePresetReactOptions | ClassicRuntimePresetReactOptions;
type RuleCondition = string | RegExp | (string | RegExp)[];
type BabelConfigUtils = {
    addPlugins: (plugins: PluginItem[]) => void;
    addPresets: (presets: PluginItem[]) => void;
    removePlugins: (plugins: string | string[]) => void;
    removePresets: (presets: string | string[]) => void;
    modifyPresetEnvOptions: (options: PresetEnvOptions) => void;
    modifyPresetReactOptions: (options: PresetReactOptions) => void;
    /**
     * use `source.include` instead
     * @deprecated
     */
    addIncludes: (includes: RuleCondition) => void;
    /**
     * use `source.exclude` instead
     * @deprecated
     */
    addExcludes: (excludes: RuleCondition) => void;
};
type PluginBabelOptions = {
    /**
     * Used to specify the files that need to be compiled by Babel.
     */
    include?: RuleCondition;
    /**
     * Used to specify the files that do not need to be compiled by Babel.
     */
    exclude?: RuleCondition;
    /**
     * Options passed to `babel-loader`.
     * @see https://github.com/babel/babel-loader
     */
    babelLoaderOptions?: ChainedConfigWithUtils<TransformOptions, BabelConfigUtils>;
};

declare const getDefaultBabelOptions: () => {
    babelrc: boolean;
    configFile: boolean;
    compact: boolean;
    plugins: never[];
    presets: (string | {
        allowNamespaces: boolean;
        allExtensions: boolean;
        allowDeclareFields: boolean;
        optimizeConstEnums: boolean;
        isTSX: boolean;
    })[][];
};

declare const pluginBabel: (options?: PluginBabelOptions) => RsbuildPlugin;

declare const getBabelUtils: (config: TransformOptions) => BabelConfigUtils;
declare const getUseBuiltIns: (config: NormalizedConfig) => false | "usage" | "entry";
declare const modifyBabelLoaderOptions: ({ chain, CHAIN_ID, modifier, }: {
    chain: BundlerChain;
    CHAIN_ID: ChainIdentifier;
    modifier: (config: TransformOptions) => TransformOptions;
}) => void;

export { type BabelConfigUtils, type PluginBabelOptions, type PresetEnvBuiltIns, type PresetEnvOptions, type PresetEnvTargets, getBabelUtils, getDefaultBabelOptions, getUseBuiltIns, modifyBabelLoaderOptions, pluginBabel };
