"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  getBabelUtils: () => getBabelUtils,
  getDefaultBabelOptions: () => getDefaultBabelOptions,
  getUseBuiltIns: () => getUseBuiltIns,
  modifyBabelLoaderOptions: () => modifyBabelLoaderOptions,
  pluginBabel: () => pluginBabel
});
module.exports = __toCommonJS(src_exports);

// src/plugin.ts
var import_node_path2 = __toESM(require("path"));
var import_core = require("@rsbuild/core");
var import_shared2 = require("@rsbuild/shared");

// src/helper.ts
var import_node_path = require("path");
var import_shared = require("@rsbuild/shared");
var import_upath = __toESM(require("upath"));
var BABEL_JS_RULE = "babel-js";
var normalizeToPosixPath = (p) => import_upath.default.normalizeSafe((0, import_node_path.normalize)(p || "")).replace(/^([a-zA-Z]+):/, (_, m) => `/${m.toLowerCase()}`);
var formatPath = (originPath) => {
  if ((0, import_node_path.isAbsolute)(originPath)) {
    return originPath.split(import_node_path.sep).join("/");
  }
  return originPath;
};
var getPluginItemName = (item) => {
  if (typeof item === "string") {
    return formatPath(item);
  }
  if (Array.isArray(item) && typeof item[0] === "string") {
    return formatPath(item[0]);
  }
  return null;
};
var addPlugins = (plugins, config) => {
  if (config.plugins) {
    config.plugins.push(...plugins);
  } else {
    config.plugins = plugins;
  }
};
var addPresets = (presets, config) => {
  if (config.presets) {
    config.presets.push(...presets);
  } else {
    config.presets = presets;
  }
};
var removePlugins = (plugins, config) => {
  if (!config.plugins) {
    return;
  }
  const removeList = (0, import_shared.castArray)(plugins);
  config.plugins = config.plugins.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var removePresets = (presets, config) => {
  if (!config.presets) {
    return;
  }
  const removeList = (0, import_shared.castArray)(presets);
  config.presets = config.presets.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var modifyPresetOptions = (presetName, options, presets = []) => {
  presets.forEach((preset, index) => {
    if (Array.isArray(preset)) {
      if (typeof preset[0] === "string" && normalizeToPosixPath(preset[0]).includes(presetName)) {
        preset[1] = {
          ...preset[1] || {},
          ...options
          // `options` is specific to different presets
        };
      }
    } else if (typeof preset === "string" && normalizeToPosixPath(preset).includes(presetName)) {
      presets[index] = [preset, options];
    }
  });
};
var getBabelUtils = (config) => {
  const noop = () => {
  };
  return {
    addPlugins: (plugins) => addPlugins(plugins, config),
    addPresets: (presets) => addPresets(presets, config),
    removePlugins: (plugins) => removePlugins(plugins, config),
    removePresets: (presets) => removePresets(presets, config),
    // `addIncludes` and `addExcludes` are noop functions by default,
    // It can be overridden by `extraBabelUtils`.
    addIncludes: noop,
    addExcludes: noop,
    // Compat `presetEnvOptions` and `presetReactOptions` in Modern.js
    modifyPresetEnvOptions: (options) => modifyPresetOptions("@babel/preset-env", options, config.presets || []),
    modifyPresetReactOptions: (options) => modifyPresetOptions("@babel/preset-react", options, config.presets || [])
  };
};
var applyUserBabelConfig = (defaultOptions, userBabelConfig, extraBabelUtils) => {
  if (userBabelConfig) {
    const babelUtils = {
      ...getBabelUtils(defaultOptions),
      ...extraBabelUtils
    };
    return (0, import_shared.mergeChainedOptions)({
      defaults: defaultOptions,
      options: userBabelConfig,
      utils: babelUtils
    });
  }
  return defaultOptions;
};
var getUseBuiltIns = (config) => {
  const { polyfill } = config.output;
  if (polyfill === "ua" || polyfill === "off") {
    return false;
  }
  return polyfill;
};
var modifyBabelLoaderOptions = ({
  chain,
  CHAIN_ID,
  modifier
}) => {
  const ruleIds = [CHAIN_ID.RULE.JS, CHAIN_ID.RULE.JS_DATA_URI, BABEL_JS_RULE];
  ruleIds.forEach((ruleId) => {
    if (chain.module.rules.has(ruleId)) {
      const rule = chain.module.rule(ruleId);
      if (rule.uses.has(CHAIN_ID.USE.BABEL)) {
        rule.use(CHAIN_ID.USE.BABEL).tap(modifier);
      }
    }
  });
};

// src/plugin.ts
var DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS = {
  allowNamespaces: true,
  allExtensions: true,
  allowDeclareFields: true,
  // aligns Babel's behavior with TypeScript's default behavior.
  // https://babeljs.io/docs/en/babel-preset-typescript#optimizeconstenums
  optimizeConstEnums: true,
  isTSX: true
};
var getDefaultBabelOptions = () => {
  return {
    babelrc: false,
    configFile: false,
    compact: (0, import_shared2.isProd)(),
    plugins: [],
    presets: [
      // TODO: only apply preset-typescript for ts file (isTSX & allExtensions false)
      [
        require.resolve("@babel/preset-typescript"),
        DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS
      ]
    ]
  };
};
var pluginBabel = (options = {}) => ({
  name: import_core.PLUGIN_BABEL_NAME,
  setup(api) {
    api.modifyBundlerChain(async (chain, { CHAIN_ID }) => {
      const getBabelOptions = () => {
        const baseConfig = getDefaultBabelOptions();
        const userBabelConfig = applyUserBabelConfig(
          (0, import_shared2.cloneDeep)(baseConfig),
          options.babelLoaderOptions
        );
        return userBabelConfig;
      };
      const babelOptions = getBabelOptions();
      const babelLoader = import_node_path2.default.resolve(
        __dirname,
        "../compiled/babel-loader/index.js"
      );
      const { include, exclude } = options;
      if (include || exclude) {
        const rule = chain.module.rule(BABEL_JS_RULE);
        if (include) {
          (0, import_shared2.castArray)(include).forEach((condition) => {
            rule.include.add(condition);
          });
        }
        if (exclude) {
          (0, import_shared2.castArray)(exclude).forEach((condition) => {
            rule.exclude.add(condition);
          });
        }
        const swcRule = chain.module.rules.get(CHAIN_ID.RULE.JS).use(CHAIN_ID.USE.SWC);
        const swcLoader = swcRule.get("loader");
        const swcOptions = swcRule.get("options");
        rule.test(import_shared2.SCRIPT_REGEX).use(CHAIN_ID.USE.SWC).loader(swcLoader).options(swcOptions).end().use(CHAIN_ID.USE.BABEL).loader(babelLoader).options(babelOptions);
      } else {
        const rule = chain.module.rule(CHAIN_ID.RULE.JS);
        rule.test(import_shared2.SCRIPT_REGEX).use(CHAIN_ID.USE.BABEL).after(CHAIN_ID.USE.SWC).loader(babelLoader).options(babelOptions);
      }
    });
  }
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getBabelUtils,
  getDefaultBabelOptions,
  getUseBuiltIns,
  modifyBabelLoaderOptions,
  pluginBabel
});
