import { createRequire } from 'module';
var require = createRequire(import.meta['url']);

var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});

// ../../node_modules/.pnpm/@modern-js+module-tools@2.46.1_typescript@5.3.2/node_modules/@modern-js/module-tools/shims/esm.js
import { fileURLToPath } from "url";
import path from "path";
var getFilename = () => fileURLToPath(import.meta.url);
var getDirname = () => path.dirname(getFilename());
var __dirname = /* @__PURE__ */ getDirname();

// src/plugin.ts
import path2 from "path";
import { PLUGIN_BABEL_NAME } from "@rsbuild/core";
import { castArray as castArray2, cloneDeep, SCRIPT_REGEX, isProd } from "@rsbuild/shared";

// src/helper.ts
import { isAbsolute, normalize, sep } from "path";
import {
  castArray,
  mergeChainedOptions
} from "@rsbuild/shared";
import upath from "upath";
var BABEL_JS_RULE = "babel-js";
var normalizeToPosixPath = (p) => upath.normalizeSafe(normalize(p || "")).replace(/^([a-zA-Z]+):/, (_, m) => `/${m.toLowerCase()}`);
var formatPath = (originPath) => {
  if (isAbsolute(originPath)) {
    return originPath.split(sep).join("/");
  }
  return originPath;
};
var getPluginItemName = (item) => {
  if (typeof item === "string") {
    return formatPath(item);
  }
  if (Array.isArray(item) && typeof item[0] === "string") {
    return formatPath(item[0]);
  }
  return null;
};
var addPlugins = (plugins, config) => {
  if (config.plugins) {
    config.plugins.push(...plugins);
  } else {
    config.plugins = plugins;
  }
};
var addPresets = (presets, config) => {
  if (config.presets) {
    config.presets.push(...presets);
  } else {
    config.presets = presets;
  }
};
var removePlugins = (plugins, config) => {
  if (!config.plugins) {
    return;
  }
  const removeList = castArray(plugins);
  config.plugins = config.plugins.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var removePresets = (presets, config) => {
  if (!config.presets) {
    return;
  }
  const removeList = castArray(presets);
  config.presets = config.presets.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var modifyPresetOptions = (presetName, options, presets = []) => {
  presets.forEach((preset, index) => {
    if (Array.isArray(preset)) {
      if (typeof preset[0] === "string" && normalizeToPosixPath(preset[0]).includes(presetName)) {
        preset[1] = {
          ...preset[1] || {},
          ...options
          // `options` is specific to different presets
        };
      }
    } else if (typeof preset === "string" && normalizeToPosixPath(preset).includes(presetName)) {
      presets[index] = [preset, options];
    }
  });
};
var getBabelUtils = (config) => {
  const noop = () => {
  };
  return {
    addPlugins: (plugins) => addPlugins(plugins, config),
    addPresets: (presets) => addPresets(presets, config),
    removePlugins: (plugins) => removePlugins(plugins, config),
    removePresets: (presets) => removePresets(presets, config),
    // `addIncludes` and `addExcludes` are noop functions by default,
    // It can be overridden by `extraBabelUtils`.
    addIncludes: noop,
    addExcludes: noop,
    // Compat `presetEnvOptions` and `presetReactOptions` in Modern.js
    modifyPresetEnvOptions: (options) => modifyPresetOptions("@babel/preset-env", options, config.presets || []),
    modifyPresetReactOptions: (options) => modifyPresetOptions("@babel/preset-react", options, config.presets || [])
  };
};
var applyUserBabelConfig = (defaultOptions, userBabelConfig, extraBabelUtils) => {
  if (userBabelConfig) {
    const babelUtils = {
      ...getBabelUtils(defaultOptions),
      ...extraBabelUtils
    };
    return mergeChainedOptions({
      defaults: defaultOptions,
      options: userBabelConfig,
      utils: babelUtils
    });
  }
  return defaultOptions;
};
var getUseBuiltIns = (config) => {
  const { polyfill } = config.output;
  if (polyfill === "ua" || polyfill === "off") {
    return false;
  }
  return polyfill;
};
var modifyBabelLoaderOptions = ({
  chain,
  CHAIN_ID,
  modifier
}) => {
  const ruleIds = [CHAIN_ID.RULE.JS, CHAIN_ID.RULE.JS_DATA_URI, BABEL_JS_RULE];
  ruleIds.forEach((ruleId) => {
    if (chain.module.rules.has(ruleId)) {
      const rule = chain.module.rule(ruleId);
      if (rule.uses.has(CHAIN_ID.USE.BABEL)) {
        rule.use(CHAIN_ID.USE.BABEL).tap(modifier);
      }
    }
  });
};

// src/plugin.ts
var DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS = {
  allowNamespaces: true,
  allExtensions: true,
  allowDeclareFields: true,
  // aligns Babel's behavior with TypeScript's default behavior.
  // https://babeljs.io/docs/en/babel-preset-typescript#optimizeconstenums
  optimizeConstEnums: true,
  isTSX: true
};
var getDefaultBabelOptions = () => {
  return {
    babelrc: false,
    configFile: false,
    compact: isProd(),
    plugins: [],
    presets: [
      // TODO: only apply preset-typescript for ts file (isTSX & allExtensions false)
      [
        __require.resolve("@babel/preset-typescript"),
        DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS
      ]
    ]
  };
};
var pluginBabel = (options = {}) => ({
  name: PLUGIN_BABEL_NAME,
  setup(api) {
    api.modifyBundlerChain(async (chain, { CHAIN_ID }) => {
      const getBabelOptions = () => {
        const baseConfig = getDefaultBabelOptions();
        const userBabelConfig = applyUserBabelConfig(
          cloneDeep(baseConfig),
          options.babelLoaderOptions
        );
        return userBabelConfig;
      };
      const babelOptions = getBabelOptions();
      const babelLoader = path2.resolve(
        __dirname,
        "../compiled/babel-loader/index.js"
      );
      const { include, exclude } = options;
      if (include || exclude) {
        const rule = chain.module.rule(BABEL_JS_RULE);
        if (include) {
          castArray2(include).forEach((condition) => {
            rule.include.add(condition);
          });
        }
        if (exclude) {
          castArray2(exclude).forEach((condition) => {
            rule.exclude.add(condition);
          });
        }
        const swcRule = chain.module.rules.get(CHAIN_ID.RULE.JS).use(CHAIN_ID.USE.SWC);
        const swcLoader = swcRule.get("loader");
        const swcOptions = swcRule.get("options");
        rule.test(SCRIPT_REGEX).use(CHAIN_ID.USE.SWC).loader(swcLoader).options(swcOptions).end().use(CHAIN_ID.USE.BABEL).loader(babelLoader).options(babelOptions);
      } else {
        const rule = chain.module.rule(CHAIN_ID.RULE.JS);
        rule.test(SCRIPT_REGEX).use(CHAIN_ID.USE.BABEL).after(CHAIN_ID.USE.SWC).loader(babelLoader).options(babelOptions);
      }
    });
  }
});
export {
  getBabelUtils,
  getDefaultBabelOptions,
  getUseBuiltIns,
  modifyBabelLoaderOptions,
  pluginBabel
};
